%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 
% Goal: Compare the proposed algorithm with other algorithms
%
% Author: Geethu Joseph, ECE Dept., IISc
% Email : geethu@ece.iisc.ernet.in
% Date last updated : 13 May. 2016
%
%
% Candidate algorithms:
%   1. N. Vaswani, Kalman Filtered Compressed Sensing, ICIP 2008
%   2. G. Joseph, Online Recovery of Temporally Correlated Sparse
%       Signals Using Multiple Measurement Vectors, Globecom 2015
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;  
home;

addpath(genpath('Other_Algos'))
FOLDER = 'Temp_Plots/Highly_Correlated';
OUTFILE = fullfile(FOLDER, 'Compare_K_MSBL.mat');

RMSE_FIG    = fullfile(FOLDER, 'Compare_RMSE_K_MSBL.fig');
SUPP_FIG    = fullfile(FOLDER, 'Compare_Supp_K_MSBL.fig');
TIME_FIG    = fullfile(FOLDER, 'Compare_Time_K_MSBL.fig');

RMSE_EPS    = fullfile(FOLDER, 'Compare_RMSE_K_MSBL.eps');
SUPP_EPS    = fullfile(FOLDER, 'Compare_Supp_K_MSBL.eps');
TIME_EPS    = fullfile(FOLDER, 'Compare_Time_K_MSBL.eps');

%% Control Parameters

ITER_NUM    = 100;       % Trial number (i.e. number of repeating the experiment)
MES_LEN     = 20;       % Row number of the dictionary matrix 
SPARSE_LEN  = 60;       % Column number of the dictionary matrix
SPARSITY    = 6;        % Number of nonzero rows (i.e. source number) in the solution matrix

PRINT       = 1;
SPARSE_NUM  = 30: 30 : 300;    % Number of measurement vectors
DELAY       = 0;                % Processing Delay
THRESHOLD   = 10^-2;
OUT_LEN     = 1;

SNR         = 20;
CORR_MIN    = 0.9;

%% Counting Variables
ALGOS = 8;
lt_K    = length(SPARSE_NUM);

Time_K_MSBL    = zeros(lt_K,ALGOS);
SuppRec_K_MSBL = zeros(lt_K,ALGOS);
RMSE_K_MSBL    = zeros(lt_K,ALGOS);

%% Iterations

h = waitbar(0,'Please wait...');
fprintf('\n Varying K \n');

fprintf('\n************************PARAMETERS************************\n');
fprintf(...
    ' Measurements \t = %d \n Lt of unknown \t = %d \n Sparsity \t = %d \n '...
    , MES_LEN , SPARSE_LEN, SPARSITY );



for iter = 1 : ITER_NUM
    if PRINT == 1
    	fprintf('\n \n\nTrial #%d \n',iter);
	end;

	% Generate dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
	A = randn(MES_LEN, SPARSE_LEN);
    A = A./(ones(MES_LEN,1)*sqrt(sum(A.^2)));

    % Generate the K nonzero rows     
	D_temp = 1-rand(SPARSITY,1)*(1-CORR_MIN);            % true temporal correlations of each source
    nonzeroX(:,1) = randn(SPARSITY,1);
	for i = 2 : SPARSE_NUM(lt_K)*100
        nonzeroX(:,i) = D_temp .* nonzeroX(:,i-1) + sqrt(1-D_temp.^2).*randn(SPARSITY,1);
	end
    
       
	% Locations of nonzero rows are randomly chosen
    ind = randperm(SPARSE_LEN);
	indice = ind(1:SPARSITY);
    Xgen = zeros(SPARSE_LEN, SPARSE_NUM(lt_K));
	D = 1-rand(SPARSE_LEN,1)*CORR_MIN;         
        
    Xgen(indice,:) = nonzeroX(:,end-SPARSE_NUM(lt_K)+1:end);
	D(indice)    = D_temp;
    Q = eye(SPARSE_LEN) - diag(D)^2;  % covarinace of innovation term
	% Noiseless signal
    Y_Noiseless = A * Xgen;
    
	% Observation noise
    if SNR == Inf
        Y = Y_Noiseless;
        stdnoise = 0;
    else
        stdnoise = std(reshape(Y_Noiseless,MES_LEN*SPARSE_NUM(lt_K),1))*10^(-SNR/20);
        noise = randn(MES_LEN, SPARSE_NUM(lt_K)) * stdnoise;

	% Noisy signal
        Y = Y_Noiseless + noise;
	end;
    
    for iter_K = 1 : lt_K
        algo = 1;
% %% ****************** Offline MSBL ****************** 
%         
%         tic;
%         X = KMSBL(Y(:,1:SPARSE_NUM(iter_K)), A, stdnoise*eye(MES_LEN),diag(D));
%         time_elapsed = toc;
%         Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;
% 
%         %Support Recovery
%         SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
%         SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;
% 
%         %RMSE
%         perf_K_MSBL_Off = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
%         RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL_Off;
% 
%         if PRINT == 1
%             fprintf(' Offline MSBL: \n \t time = %5.2f; SuppRec = %3.2f, Ave-RMSE = %3.2f%%; Ave-Time = %4.3f \n',...
%             time_elapsed,SuppRec,RMSE_K_MSBL(iter_K)/iter*100,Time_K_MSBL(iter_K)/iter);
%         end;
% 
% 
% %% ****************** Online Sawtooth Lag MSBL ****************** 
%         algo = algo + 1;
%         tic;
%         X = Sawtooth_Online_KMSBL(Y(:,1:SPARSE_NUM(iter_K)), A, stdnoise*eye(MES_LEN), diag(D), 'delay', DELAY,...
%                 'expon',0.6,'no_init',0,'out_length',OUT_LEN);
%         time_elapsed = toc;
%         Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;
% 
%         %Support Recovery
%         SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
%         SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;
% 
%         %RMSE
%         perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
%         RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
%         if PRINT == 1
%             fprintf('\n Sawtooth Lag KMSBL: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
%             	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
%         end;
%          
%         
% %% *********************Iterative Online KMSBL ***********************
%         algo = algo + 1;
%         tic;
%         X = sawtooth_lag_rec(Y(:,1:SPARSE_NUM(iter_K)), A, D, stdnoise, 0);
%         time_elapsed = toc;
%         Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;
% 
%         % Support Recovery
%         SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
%         SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;
% 
%         % RMSE
%         perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
%         RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
%        if PRINT == 1
%             fprintf('\n Iterative Online KMSBL: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
%             	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
%         end;
        
        
%% ********************* Kalman Filtered Compressed Sensing ***********************
        algo = algo + 1;
        tic;
        X = kfcs_full(Y(:,1:SPARSE_NUM(iter_K)),Q,A,Q,stdnoise*eye(MES_LEN),diag(D));
        time_elapsed = toc;
        Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;

        % RMSE
        perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
       if PRINT == 1
            fprintf('\n Kalman Filtered Compressed Sensing: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
            	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
        end;

%% ********************* Least Squared Compressed Sensing ***********************
        algo = algo + 1;
        tic;
        X = lscs_full(Y(:,1:SPARSE_NUM(iter_K)),Q,A,Q,stdnoise*eye(MES_LEN));
        time_elapsed = toc;
        Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;

        % RMSE
        perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
       if PRINT == 1
            fprintf('\n Least Squared Compressed Sensing: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
            	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
        end;

%% ********************* Regular Compressed Sensing ***********************
        algo = algo + 1;
        tic;
        X = regularcs(Y(:,1:SPARSE_NUM(iter_K)),Q,A,Q,stdnoise*eye(MES_LEN));
        time_elapsed = toc;
        Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;

        % RMSE
        perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
       if PRINT == 1
            fprintf('\n Regular Compressed Sensing: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
            	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
        end;
        
%% ********************* Rewieghted L1 Dynamic Filtering***********************
        algo = algo + 1;
        tic;
%            RWL1_DF(y_meas,G_n, F_n, dyn_var, obs_var, num_EM, tau, beta)
        Gn = repmat(A,[1 1 SPARSE_NUM(iter_K)]);
        Fn = repmat(diag(D),[1 1 SPARSE_NUM(iter_K)]);
        X = RWL1_DF(Y(:,1:SPARSE_NUM(iter_K)),Gn,Fn,norm(diag(Q))^2/SPARSE_LEN, stdnoise^2,30);
        time_elapsed = toc;
        Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;

        % RMSE
        perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
       if PRINT == 1
            fprintf('\n Rewieghted L1 Dynamic Filtering: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
            	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
        end;
        
%% ********************* CS MUSIC ***********************
        algo = algo + 1;
        tic;
%            CSMUSIC(Y,A,s,sigma)
        [dummy X] = CSMUSIC(Y(:,1:SPARSE_NUM(iter_K)),A, 1, stdnoise);
        time_elapsed = toc;
        Time_K_MSBL(iter_K,algo) = Time_K_MSBL(iter_K,algo) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_K_MSBL(iter_K,algo) = SuppRec_K_MSBL(iter_K,algo) + SuppRec;

        % RMSE
        perf_K_MSBL = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_K_MSBL(iter_K,algo) = RMSE_K_MSBL(iter_K,algo) + perf_K_MSBL;
       if PRINT == 1
            fprintf('\n CS Music: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
            	time_elapsed,SuppRec,RMSE_K_MSBL(iter_K,algo)/iter*100, SuppRec_K_MSBL(iter_K,algo)/iter*100);
        end;


        
        status = (iter - 1 + iter_K/lt_K)/ITER_NUM;
        waitbar(status,h,sprintf('K: %3.2f %%',status*100));
        
        
        
    end
    save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', ...
    'Time_K_MSBL', 'SuppRec_K_MSBL', 'RMSE_K_MSBL');
end
    

close(h);

%% Save Results
Time_K_MSBL    = Time_K_MSBL/ITER_NUM;
SuppRec_K_MSBL = SuppRec_K_MSBL/ITER_NUM;
RMSE_K_MSBL     = RMSE_K_MSBL/ITER_NUM;

save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', ...
    'Time_K_MSBL', 'SuppRec_K_MSBL', 'RMSE_K_MSBL');

%% Plot and save figure

% colors
DarkGreen   = [0.2       0.5000    0.2   ];
LightGreen  = [0.4660    0.6740    0.1880];
LightBlue   = [0         0.8       1     ];
SkyBlue     = [0         0.4470    0.7410];
Purple      = [0.4940    0.1840    0.5560];
DarkYellow  = [0.9290    0.6940    0.1250];
Orange      = [0.8500    0.3250    0.0980];
BrickRed    = [0.6350    0.0780    0.1840];
DarkCyan    = [0         0.6       0.6   ];
Gray        = [0.55      0.55      0.55  ];

%%%%%%% RMSE
figure;
set(0,'defaulttextinterpreter','tex')
set(0,'DefaultLineLineWidth',3)
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineMarkerSize',14)
set(0,'DefaultAxesFontWeight','bold')
set(gca,'FontSize',16)
set(get(gca,'Xlabel'),'FontSize',16)
set(get(gca,'Ylabel'),'FontSize',16)
set(get(gca,'Title'),'FontSize',16)
set(get(gca,'Xlabel'),'FontWeight','bold')
set(get(gca,'Ylabel'),'FontWeight','bold')
set(get(gca,'Title'),'FontWeight','bold')
HA = semilogy(  SPARSE_NUM, RMSE_K_MSBL (:,6),'--s' ,...
                SPARSE_NUM, RMSE_K_MSBL (:,3),'-->' ,...
                SPARSE_NUM, RMSE_K_MSBL (:,7),'--d' ,...
                SPARSE_NUM, RMSE_K_MSBL (:,2),'-*',...
                SPARSE_NUM, RMSE_K_MSBL (:,5),'--v' ,...
                SPARSE_NUM, RMSE_K_MSBL (:,4),'--<' ,...
                SPARSE_NUM, RMSE_K_MSBL (:,1),'-.o',...
                SPARSE_NUM, RMSE_K_MSBL (:,8),'-.^' ...
                );
axis square;
ylabel('RMSE');
xlabel('No. of Sparse Vectors K')

set(HA(1),'color', DarkYellow);
set(HA(2),'color', Purple);            
set(HA(3),'color', DarkGreen);
set(HA(4),'color', 'm');
set(HA(5),'color', Orange);         
set(HA(6),'color', SkyBlue);
set(HA(7),'color', 'k');
set(HA(8),'color', 'g');
% legend( 'Regular l_1 norm','Iterative KMSBL','RL1-DF','Non-iterative KMSBL','LS-CS','KF-CS','Offline KMSBL',...
%                 'CS MUSIC');
% Save figures
saveas(gca,RMSE_FIG);
saveas(gca, RMSE_EPS, 'epsc');

%%%%%%% Support Recovery
figure;
set(0,'defaulttextinterpreter','tex')
HA = plot(      SPARSE_NUM, SuppRec_K_MSBL (:,8),'-.^' ,...
                SPARSE_NUM, SuppRec_K_MSBL (:,1),'-.o' ,...
                SPARSE_NUM, SuppRec_K_MSBL (:,4),'--<' ,...
                SPARSE_NUM, SuppRec_K_MSBL (:,5),'--v' ,...
                SPARSE_NUM, SuppRec_K_MSBL (:,6),'--s',...
                SPARSE_NUM, SuppRec_K_MSBL (:,2),'-*',...
                SPARSE_NUM, SuppRec_K_MSBL (:,3),'-->',...
                SPARSE_NUM, SuppRec_K_MSBL (:,7),'--d'...
                );
ylabel('SRR');
xlabel('No. of Sparse Vectors K')

set(HA(8),'color', DarkGreen);            
set(HA(7),'color', Purple);  
set(HA(6),'color', 'm');
set(HA(5),'color', DarkYellow);
set(HA(4),'color', Orange);         
set(HA(3),'color', SkyBlue);
set(HA(2),'color', 'k');
set(HA(1),'color', 'g');

%axis([-inf inf 0.5 1])
axis square;
% legend('CS MUSIC', 'Offline KMSBL','KF-CS','LS-CS','Regular l_1 Norm' ,'Non-iterative KMSBL','Iterative KMSBL', 'RL1-DF');
% Save figures
saveas(gca,SUPP_FIG);
saveas(gca, SUPP_EPS, 'epsc');

%%%%%%% Run Time
figure;
set(0,'defaulttextinterpreter','tex')
HA = plot(      SPARSE_NUM, Time_K_MSBL (:,8),'-.^' ,...
                SPARSE_NUM, Time_K_MSBL (:,1),'-.o' ,...
                SPARSE_NUM, Time_K_MSBL (:,4),'--<' ,...
                SPARSE_NUM, Time_K_MSBL (:,5),'--v' ,...
                SPARSE_NUM, Time_K_MSBL (:,6),'--s',...
                SPARSE_NUM, Time_K_MSBL (:,2),'-*',...
                SPARSE_NUM, Time_K_MSBL (:,3),'-->',...
                SPARSE_NUM, Time_K_MSBL (:,7),'--d'...
                );
            
 
set(HA(8),'color', DarkGreen);            
set(HA(7),'color', Purple);  
set(HA(6),'color', 'm');
set(HA(5),'color', DarkYellow);
set(HA(4),'color', Orange);         
set(HA(3),'color', SkyBlue);
set(HA(2),'color', 'k');
set(HA(1),'color', 'g');

axis square;
ylabel('Run Time');
xlabel('No. of Sparse Vectors K')
legend('CS MUSIC','Offline KMSBL','KF-CS','LS-CS','RL1-DF','Non-iterative KMSBL', 'Iterative KMSBL','Regular l_1 Norm');

% Save figures
saveas(gca,TIME_FIG);
saveas(gca, TIME_EPS, 'epsc');