%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 
% Goal: Performance of Online Vs Offline MSBL in noisy case
%
% Author: Geethu Joseph, ECE Dept., IISc
% Email : geethu@ece.iisc.ernet.in
% Date last updated : 18 Feb. 2016
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;  
home;

FOLDER = 'Temp_Plots/Highly_Correlated';
OUTFILE = fullfile(FOLDER, 'Delay_K_MSBL.mat');

RMSE_FIG    = fullfile(FOLDER, 'Delay_RMSE_K_MSBL.fig');
SUPP_FIG    = fullfile(FOLDER, 'Delay_Supp_K_MSBL.fig');
TIME_FIG    = fullfile(FOLDER, 'Delay_Time_K_MSBL.fig');

RMSE_EPS    = fullfile(FOLDER, 'Delay_RMSE_K_MSBL.eps');
SUPP_EPS    = fullfile(FOLDER, 'Delay_Supp_K_MSBL.eps');
TIME_EPS    = fullfile(FOLDER, 'Delay_Time_K_MSBL.eps');

%% Control Parameters

ITER_NUM    = 100;     % Trial number (i.e. number of repeating the experiment)
MES_LEN     = 20;       % Row number of the dictionary matrix 
SPARSE_LEN  = 60;       % Column number of the dictionary matrix
SPARSITY    = 6;        % Number of nonzero rows (i.e. source number) in the solution matrix

PRINT       = 1;
SPARSE_NUM  = 150;      % Number of measurement vectors
DELAY       = 5:5:50;   % Processing Delay
THRESHOLD   = 10^-2;

SNR         = 20;
OUT_LEN     = 1 : 2 : 5;
CORR_MIN    = 0.9;
%% Counting Variables
lt_delay    = length(DELAY);
lt_outLen   = length(OUT_LEN);

Time_K_MSBL_Off    = zeros(lt_delay,1);
SuppRec_K_MSBL_Off = zeros(lt_delay,1);
RMSE_K_MSBL_Off    = zeros(lt_delay,1);

% Online Sawtooth Lag MSBL Variables
Time_K_MSBL     = zeros(lt_delay,2,lt_outLen+1);
SuppRec_K_MSBL  = zeros(lt_delay,2,lt_outLen+1);
RMSE_K_MSBL     = zeros(lt_delay,2,lt_outLen+1);

%% Iterations

h = waitbar(0,'Please wait...');
fprintf('\n Varying Delay \n');

fprintf('\n************************PARAMETERS************************\n');
fprintf(...
    ' Measurements \t = %d \n Lt of unknown \t = %d \n Sparsity \t = %d \n '...
    , MES_LEN , SPARSE_LEN, SPARSITY );



for iter = 1 : ITER_NUM
    if PRINT == 1
    	fprintf('\n \n\nTrial #%d \n',iter);
	end;

	% Generate dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
        A = randn(MES_LEN, SPARSE_LEN);
        A = A./(ones(MES_LEN,1)*sqrt(sum(A.^2)));

        % Generate the K nonzero rows
        
        D_temp = ones(SPARSITY,1)*CORR_MIN;            % true temporal correlations of each source
        nonzeroX(:,1) = randn(SPARSITY,1);
        for i = 2 : SPARSE_NUM*100
            nonzeroX(:,i) = D_temp .* nonzeroX(:,i-1) + sqrt(1-D_temp.^2).*randn(SPARSITY,1);
        end
    
       
        % Locations of nonzero rows are randomly chosen
        ind = randperm(SPARSE_LEN);
        indice = ind(1:SPARSITY);
        Xgen = zeros(SPARSE_LEN, SPARSE_NUM);
        D = diag(ones(SPARSE_LEN,1)*CORR_MIN);         
        Cov = zeros(SPARSE_LEN, 1);
        Cov(indice,:) = 1;
        
        Xgen(indice,:) = nonzeroX(:,end-SPARSE_NUM+1:end);
        %D(indice)    = D_temp;
        % Noiseless signal
        Y = A * Xgen;
        if SNR ~= Inf

	        % Observation noise   
            stdnoise = std(reshape(Y,MES_LEN*SPARSE_NUM,1))*10^(-SNR/20);
            noise = randn(MES_LEN, SPARSE_NUM) * stdnoise;

            % Noisy signal
            Y = Y + noise;
        else
        	stdnoise = 0;
        end;
        
    %% ****************** Offline MSBL ****************** 
        tic;
        X = KMSBL(Y, A, stdnoise*eye(MES_LEN), D);
        time_elapsed = toc;
        Time_K_MSBL_Off = Time_K_MSBL_Off + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM);
        SuppRec_K_MSBL_Off = SuppRec_K_MSBL_Off + SuppRec;

        % RMSE
        perf_K_MSBL_Off = (norm(Xgen - X,'fro')/norm(Xgen,'fro'))^2;
        RMSE_K_MSBL_Off = RMSE_K_MSBL_Off + perf_K_MSBL_Off;

        if PRINT == 1
            fprintf(' Offline KMSBL: \n \t time = %5.2f; SuppRec = %3.2f, Ave-RMSE = %3.2f%%; Ave-Time = %4.3f \n',...
            time_elapsed,SuppRec,RMSE_K_MSBL_Off(1)/iter*100,Time_K_MSBL_Off(1)/iter);
        end;
        
    for iter_delay = 1 : lt_delay

% ****************** Online Sawtooth Lag MSBL ****************** 
        for no_init = 1 : 2
            for iter_OutLen = 1 : lt_outLen
                tic;
                X = Sawtooth_Online_KMSBL(Y, A, stdnoise*eye(MES_LEN), D, 'delay', DELAY(iter_delay),...
                    'expon',0.6,'no_init',no_init-1,'out_length',OUT_LEN(iter_OutLen));
                time_elapsed = toc;
                Time_K_MSBL (iter_delay,no_init,iter_OutLen) = Time_K_MSBL (iter_delay,no_init,iter_OutLen ) + time_elapsed;

                % Support Recovery
                SuppRec  = length(find(abs(Xgen - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM);
                SuppRec_K_MSBL (iter_delay,no_init,iter_OutLen ) = SuppRec_K_MSBL (iter_delay,no_init,iter_OutLen ) + SuppRec ;

                % RMSE
                perf_K_MSBL  = (norm(Xgen - X,'fro')/norm(Xgen,'fro'))^2;
                RMSE_K_MSBL (iter_delay,no_init,iter_OutLen ) = RMSE_K_MSBL (iter_delay,no_init,iter_OutLen ) + perf_K_MSBL ;
                if PRINT == 1
                    fprintf('\n Sawtooth Lag KMSBL: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
                            time_elapsed,SuppRec ,RMSE_K_MSBL (iter_delay,no_init,iter_OutLen)/iter*100,...
                            SuppRec_K_MSBL (iter_delay,no_init,iter_OutLen )/iter*100);
                end;
            end;
            tic;
                X = Fixed_Online_KMSBL(Y, A, stdnoise*eye(MES_LEN), D, 'delay', DELAY(iter_delay),...
                    'expon',0.6,'no_init',no_init-1);
                time_elapsed = toc;
                Time_K_MSBL (iter_delay,no_init,lt_outLen+1) = Time_K_MSBL (iter_delay,no_init,lt_outLen+1) + time_elapsed;

                % Support Recovery
                SuppRec  = length(find(abs(Xgen - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM);
                SuppRec_K_MSBL (iter_delay,no_init,lt_outLen+1 ) = SuppRec_K_MSBL (iter_delay,no_init,lt_outLen+1) + SuppRec ;

                % RMSE
                perf_K_MSBL  = (norm(Xgen - X,'fro')/norm(Xgen,'fro'))^2;
                RMSE_K_MSBL (iter_delay,no_init,lt_outLen+1 ) = RMSE_K_MSBL (iter_delay,no_init,lt_outLen+1) + perf_K_MSBL ;
                if PRINT == 1
                    fprintf('\n Fixed Lag KMSBL: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
                            time_elapsed,SuppRec ,RMSE_K_MSBL (iter_delay,no_init,lt_outLen+1)/iter*100,...
                            SuppRec_K_MSBL (iter_delay,no_init,lt_outLen+1 )/iter*100);
                end;
        end
        
        status = (iter - 1 + iter_delay/lt_delay)/ITER_NUM;
        waitbar(status,h,sprintf('Delay: %3.2f %%',status*100));
    end
    save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', ...
    'Time_K_MSBL_Off', 'SuppRec_K_MSBL_Off', 'RMSE_K_MSBL_Off', ...
    'Time_K_MSBL', 'SuppRec_K_MSBL', 'RMSE_K_MSBL','CORR');
end
    

close(h);

%% Save Results
Time_K_MSBL_Off    = Time_K_MSBL_Off/ITER_NUM;
SuppRec_K_MSBL_Off = SuppRec_K_MSBL_Off/ITER_NUM;
RMSE_K_MSBL_Off     = RMSE_K_MSBL_Off/ITER_NUM;

Time_K_MSBL     = Time_K_MSBL /ITER_NUM;
SuppRec_K_MSBL  = SuppRec_K_MSBL /ITER_NUM;
RMSE_K_MSBL      = RMSE_K_MSBL /ITER_NUM;

save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', 'OUT_LEN', ...
    'Time_K_MSBL_Off', 'SuppRec_K_MSBL_Off', 'RMSE_K_MSBL_Off', ...
    'Time_K_MSBL', 'SuppRec_K_MSBL', 'RMSE_K_MSBL','CORR');

%% Plot and save figure

% colors
DarkGreen   = [0.2       0.5000    0.2   ];
LightGreen  = [0.4660    0.6740    0.1880];
LightBlue   = [0         0.8       1     ];
SkyBlue     = [0         0.4470    0.7410];
Purple      = [0.4940    0.1840    0.5560];
DarkYellow  = [0.9290    0.6940    0.1250];
Orange      = [0.8500    0.3250    0.0980];
BrickRed    = [0.6350    0.0780    0.1840];
DarkCyan    = [0         0.6       0.6   ];
Gray        = [0.55      0.55      0.55  ];

%%%%%%% RMSE
figure;
set(0,'defaulttextinterpreter','tex')
set(0,'DefaultLineLineWidth',3)
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineMarkerSize',14)
set(0,'DefaultAxesFontWeight','bold')
set(gca,'FontSize',16)
set(get(gca,'Xlabel'),'FontSize',16)
set(get(gca,'Ylabel'),'FontSize',16)
set(get(gca,'Title'),'FontSize',16)
set(get(gca,'Xlabel'),'FontWeight','bold')
set(get(gca,'Ylabel'),'FontWeight','bold')
set(get(gca,'Title'),'FontWeight','bold')
HA = semilogy(  DELAY, RMSE_K_MSBL (:,2,3),'-x' ,...
                DELAY, RMSE_K_MSBL (:,2,2),'--d' ,...
                DELAY, RMSE_K_MSBL (:,2,4),'-o',...
                DELAY, RMSE_K_MSBL (:,2,1),':s',...
                DELAY, RMSE_K_MSBL (:,1,3),'->' ,...
                DELAY, RMSE_K_MSBL (:,1,2),'--<' ,...
                DELAY, RMSE_K_MSBL (:,1,4),'-*',...
                DELAY, RMSE_K_MSBL (:,1,1),':v',...
                DELAY, RMSE_K_MSBL_Off,'-.'...
                );
axis square;
ylabel('RMSE');
xlabel('Maximum Delay \Delta')
% h = legend( '\textbf{No Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Offline}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
          
set(HA(1),'color', SkyBlue);
set(HA(2),'color', Orange);         
set(HA(3),'color', 'm');  
set(HA(4),'color', DarkGreen);
set(HA(5),'color', DarkYellow);
set(HA(6),'color', Purple);
set(HA(7),'color', LightGreen);
set(HA(8),'color', BrickRed);  
set(HA(9),'color', 'k'); 
% Save figures
saveas(gca,RMSE_FIG);
saveas(gca, RMSE_EPS, 'epsc');

%%%%%%% Support Recovery
figure;
set(0,'defaulttextinterpreter','tex')
HA = semilogy(  DELAY, SuppRec_K_MSBL_Off,'-.',...
                DELAY, SuppRec_K_MSBL (:,1,1),':v' ,...
                DELAY, SuppRec_K_MSBL (:,1,4),'-*' ,...
                DELAY, SuppRec_K_MSBL (:,1,2),'--<',...
                DELAY, SuppRec_K_MSBL (:,1,3),'->',...
                DELAY, SuppRec_K_MSBL (:,2,1),':s' ,...
                DELAY, SuppRec_K_MSBL (:,2,4),'-o' ,...
                DELAY, SuppRec_K_MSBL (:,2,2),'--d',...
                DELAY, SuppRec_K_MSBL (:,2,3),'-x');
axis square;
ylabel('SRR');
xlabel('Maximum Delay \Delta')
% h = legend( '\textbf{Offline}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 5}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
set(HA(9),'color', SkyBlue);
set(HA(8),'color', Orange);         
set(HA(7),'color', 'm');  
set(HA(6),'color', DarkGreen);
set(HA(5),'color', DarkYellow);
set(HA(4),'color', Purple);
set(HA(3),'color', LightGreen);
set(HA(2),'color', BrickRed);  
set(HA(1),'color', 'k');  
% Save figures
saveas(gca,SUPP_FIG);
saveas(gca, SUPP_EPS, 'epsc');

%%%%%%% Run Time
figure;
set(0,'defaulttextinterpreter','tex')

HA = semilogy(  DELAY, Time_K_MSBL_Off,'-.',...
                DELAY, Time_K_MSBL (:,1,1),':v' ,...
                DELAY, Time_K_MSBL (:,1,4),'-*' ,...
                DELAY, Time_K_MSBL (:,1,2),'--<',...
                DELAY, Time_K_MSBL (:,1,3),'->',...
                DELAY, Time_K_MSBL (:,2,1),':s' ,...
                DELAY, Time_K_MSBL (:,2,4),'-o' ,...
                DELAY, Time_K_MSBL (:,2,2),'--d',...
                DELAY, Time_K_MSBL (:,2,3),'-x');
            
set(HA(9),'color', SkyBlue);
set(HA(8),'color', Orange);         
set(HA(7),'color', 'm');  
set(HA(6),'color', DarkGreen);
set(HA(5),'color', DarkYellow);
set(HA(4),'color', Purple);
set(HA(3),'color', LightGreen);
set(HA(2),'color', BrickRed);  
set(HA(1),'color', 'k');  
axis square;
ylabel('Run Time');
xlabel('Maximum Delay \Delta')
% h = legend( '\textbf{Offline}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 5}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
% Save figures
saveas(gca,TIME_FIG);
saveas(gca, TIME_EPS, 'epsc');
