%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 
% Goal: Demonstrate the convergence of iterates of Online KMSBL
%
% Author: Geethu Joseph, ECE Dept., IISc
% Email : geethu@ece.iisc.ernet.in
% Date last updated : 24 Feb. 2016
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;  
home;

FOLDER = 'Temp_Plots/Uncorrelated';
OUTFILE         = fullfile(FOLDER, 'Convergence_MSBL_06.mat');
CONV_FIG        = fullfile(FOLDER, 'Convergence_MSBL_06.fig');
CONV_EPS        = fullfile(FOLDER, 'Convergence_MSBL_06.eps');


%% Control Parameters
MAX_ITER    = 100;
MES_LEN     = 20;       % Row number of the dictionary matrix 
SPARSE_LEN  = 60;       % Column number of the dictionary matrix
SPARSITY    = 6;        % Number of nonzero rows (i.e. source number) in the solution matrix
SNR         = 20;
SPARSE_NUM  = 300;

DELAY       = [5 10 15];       % Processing Delay
OUT_LEN     = 3;
FIXED_SAW   = 1;

EXPONENT    = 0.6;
DS          = round(25/OUT_LEN);
%% Counting Variables

lt_DEL = length(DELAY);

% Error in Covarince KMSBL Variables
xval           = [DELAY(1)+1:OUT_LEN:SPARSE_NUM+OUT_LEN-1]';
Err_KMSBL_On   = zeros(length(xval),2*lt_DEL);
FitCurve       = zeros(length(xval), 2*lt_DEL);

NonzeroX       = zeros (SPARSITY,SPARSE_NUM, 2);
Para           = zeros(2,2*lt_DEL);
%% Iterations

h = waitbar(0,'Please wait...');
fprintf('\n NOISY CASE: convergence trajectory \n');

fprintf('\n************************PARAMETERS************************\n');
fprintf(...
    ' Measurements \t = %d \n Lt of unknown \t = %d \n Sparsity \t = %d \n '...
    , MES_LEN , SPARSE_LEN, SPARSITY );
for iter = 1 : MAX_ITER
       
        % Generate dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
        A = randn(MES_LEN, SPARSE_LEN);
        A = A./(ones(MES_LEN,1)*sqrt(sum(A.^2)));

        % Generate the K nonzero rows     
        nonzeroX(:,:,1) = 2*(rand(SPARSITY,SPARSE_NUM)>0.5)-1;
        nonzeroX(:,:,2) = randn(SPARSITY,SPARSE_NUM);
        
        % Locations of nonzero rows are randomly chosen
        ind = randperm(SPARSE_LEN);
        indice = ind(1:SPARSITY);
        Xgen = zeros(SPARSE_LEN, SPARSE_NUM);
                
        Cov = zeros(SPARSE_LEN, 1);
        Cov(indice,:) = 1;
        
        for distri = 1 : 2
            Xgen(indice,:) = nonzeroX(:,:,distri);
            % Noiseless signal
            Y = A * Xgen;
            if SNR ~= Inf

                % Observation noise   
                stdnoise = std(reshape(Y,MES_LEN*SPARSE_NUM,1))*10^(-SNR/20);
                noise = randn(MES_LEN, SPARSE_NUM) * stdnoise;

                % Noisy signal
                Y = Y + noise;
            else
                stdnoise = 0;
            end;

            for iter_del = 1 : lt_DEL
                curr_ind = iter_del + (distri-1)*lt_DEL;
                            % Online_KMSBL(A, Y, R, D, varargin)
                [X, err] = Online_MSBL(Y, A, stdnoise, ...
                                        'delay', DELAY(iter_del), 'out_length', OUT_LEN, 'trueval',Cov,...
                                        'no_init', 1, 'expon',EXPONENT);
                Err_KMSBL_On(1:length(err),curr_ind) = Err_KMSBL_On(1:length(err),curr_ind) + err;
            end;
        end;
        status = iter/MAX_ITER;
        waitbar(status,h,sprintf('Convergence: %3.2f %%',status*100));
        
end;
close (h);

        Err_KMSBL_On = Err_KMSBL_On/MAX_ITER;
        % fit power curve
        for iter = 1 : 2*lt_DEL
            Para(:,iter) = polyfit(log(xval),log(Err_KMSBL_On(:,iter)),1); 
            FitCurve(:,iter) = exp(Para(2,iter))*xval.^Para(1,iter); 
        end;
        
        save(OUTFILE,  'MES_LEN', 'SPARSE_LEN','SPARSITY', 'DELAY', 'SPARSE_NUM', ...
            'Err_KMSBL_On','FitCurve');

%% Plot figure

Hfig = figure;
%set(Hfig, 'Position', [250    212    400    400]);
DarkGreen   = [  0       0.5000       0  ];
LightGreen  = [0.4660    0.6740    0.1880];
LightBlue   = [   0        0.8        1  ];
SkyBlue     = [  0       0.4470    0.7410];
Purple      = [0.4940    0.1840    0.5560];
DarkYellow  = [0.9290    0.6940    0.1250];
Orange      = [0.8500    0.3250    0.0980];
BrickRed    = [0.6350    0.0780    0.1840];
DarkCyan    = [   0       0.6        0.6 ];
Gray        = 0.55*[1 1 1];

set(0,'defaulttextinterpreter','tex')

set(0,'DefaultLineLineWidth',3)
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineMarkerSize',14)
set(0,'DefaultAxesFontWeight','bold')
set(gca,'FontSize',16)
set(get(gca,'Xlabel'),'FontSize',16)
set(get(gca,'Ylabel'),'FontSize',16)
set(get(gca,'Title'),'FontSize',16)
set(get(gca,'Xlabel'),'FontWeight','bold')
set(get(gca,'Ylabel'),'FontWeight','bold')
set(get(gca,'Title'),'FontWeight','bold')



XVAL = downsample(xval,DS);
HA = semilogy(xval, FitCurve(:,1), '-', xval, FitCurve(:,4), '--',...
              XVAL, downsample(Err_KMSBL_On(:,1),DS),'x', XVAL, downsample(Err_KMSBL_On(:,4),DS),'o',...
              xval, FitCurve(:,2), '-', xval, FitCurve(:,5), '--',...
              XVAL, downsample(Err_KMSBL_On(:,2),DS),'x', XVAL, downsample(Err_KMSBL_On(:,5),DS),'o',...
              xval, FitCurve(:,3), '-', xval, FitCurve(:,6), '--',...
              XVAL, downsample(Err_KMSBL_On(:,3),DS),'x', XVAL, downsample(Err_KMSBL_On(:,6),DS),'o');
          
axis square;
ylabel('Error in Itertate');
xlabel('Number of Sparse Vectors K');
axis([0 300 10^-1 10^3]);

% set(HA(1),'color', LightGreen);     set(HA(2),'color', DarkGreen);
% set(HA(3),'color', 'r');            set(HA(4),'color', BrickRed);
% 
% set(HA(5),'color', LightBlue);      set(HA(6),'color', SkyBlue);
% set(HA(7),'color', BrickRed);       set(HA(8),'color', Orange);
% 
% 
% 
% set(HA(9),'color',  'g');           set(HA(10),'color', Purple);
% set(HA(11),'color', 'm');           set(HA(12),'color', DarkYellow);

set(HA([1 5 9]),'color', LightGreen);      set(HA([2 6 10]),'color',  BrickRed);
set(HA([3 7 11]),'color',SkyBlue);       set(HA([4 8 12]),'color', DarkYellow);

legend( 'Fit: Rademacher',  'Error: Rademacher',...
        'Fit: Gaussian',  'Error: Gaussian');

% set(HA([1 5 9]),'color',  'g');     set(HA([2 6 10]),'color', Purple);
% set(HA([3 7 11]),'color', 'r');     set(HA([4 8 12]),'color', DarkYellow);
%legend( 'Error:  Rademacher',  'Fit: Rademacher',...
%         'Error: Gaussian',  'Fit: Gaussian')

saveas(gca, CONV_FIG);
saveas(gca, CONV_EPS, 'epsc');