%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 
% Goal: Performance of Online Vs Offline MSBL in noisy case
%
% Author: Geethu Joseph, ECE Dept., IISc
% Email : geethu@ece.iisc.ernet.in
% Date last updated : 18 Feb. 2016
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all;  
home;

FOLDER = 'Temp_Plots\Uncorrelated';
OUTFILE = fullfile(FOLDER, 'SparseNum_MSBL.mat');

RMSE_FIG    = fullfile(FOLDER, 'SparseNum_RMSE_MSBL.fig');
SUPP_FIG    = fullfile(FOLDER, 'SparseNum_Supp_MSBL.fig');
TIME_FIG    = fullfile(FOLDER, 'SparseNum_Time_MSBL.fig');

RMSE_EPS    = fullfile(FOLDER, 'SparseNum_RMSE_MSBL.eps');
SUPP_EPS    = fullfile(FOLDER, 'SparseNum_Supp_MSBL.eps');
TIME_EPS    = fullfile(FOLDER, 'SparseNum_Time_MSBL.eps');

%% Control Parameters

ITER_NUM    = 1;        % Trial number (i.e. number of repeating the experiment)
MES_LEN     = 20;       % Row number of the dictionary matrix 
SPARSE_LEN  = 60;       % Column number of the dictionary matrix
SPARSITY    = 6;        % Number of nonzero rows (i.e. source number) in the solution matrix

PRINT       = 1;
SPARSE_NUM  = 30 : 30 : 300;    % Number of measurement vectors
DELAY       = 5;                % Processing Delay
THRESHOLD   = 10^-2;

SNR         = 20;
OUT_LEN     = 1 : 2 : 5;
%% Counting Variables
lt_K    = length(SPARSE_NUM);
lt_outLen   = length(OUT_LEN);

Time_MSBL_Off    = zeros(lt_K,1);
SuppRec_MSBL_Off = zeros(lt_K,1);
RMSE_MSBL_Off    = zeros(lt_K,1);

% Online Sawtooth Lag MSBL Variables
Time_MSBL_Saw    = zeros(lt_K,2,lt_outLen);
SuppRec_MSBL_Saw = zeros(lt_K,2,lt_outLen);
RMSE_MSBL_Saw    = zeros(lt_K,2,lt_outLen);

%% Iterations

h = waitbar(0,'Please wait...');
fprintf('\n Varying K \n');

fprintf('\n************************PARAMETERS************************\n');
fprintf(...
    ' Measurements \t = %d \n Lt of unknown \t = %d \n Sparsity \t = %d \n '...
    , MES_LEN , SPARSE_LEN, SPARSITY );



for iter = 1 : ITER_NUM
    if PRINT == 1
    	fprintf('\n \n\nTrial #%d \n',iter);
	end;

	% Generate dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
	A = randn(MES_LEN, SPARSE_LEN);
    A = A./(ones(MES_LEN,1)*sqrt(sum(A.^2)));

    % Generate the K nonzero rows     
	nonzeroX = randn(SPARSITY,SPARSE_NUM(lt_K));

    % Locations of nonzero rows are randomly chosen
	ind = randperm(SPARSE_LEN);
    indice = ind(1:SPARSITY);
    Xgen = zeros(SPARSE_LEN, SPARSE_NUM(lt_K));
    Xgen(indice,:) = nonzeroX;

	% Noiseless signal
    Y_Noiseless = A * Xgen;
	% Observation noise
    if SNR == Inf
        Y = Y_Noiseless;
        stdnoise = 0;
    else
        stdnoise = std(reshape(Y_Noiseless,MES_LEN*SPARSE_NUM(lt_K),1))*10^(-SNR/20);
        noise = randn(MES_LEN, SPARSE_NUM(lt_K)) * stdnoise;

	% Noisy signal
        Y = Y_Noiseless + noise;
	end;
    for iter_K = 1 : lt_K
%% ****************** Offline MSBL ****************** 
        tic;
        X = MSBL(Y(:,1:SPARSE_NUM(iter_K)), A, stdnoise*eye(MES_LEN));
        time_elapsed = toc;
        Time_MSBL_Off(iter_K) = Time_MSBL_Off(iter_K) + time_elapsed;

        % Support Recovery
        SuppRec = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
        SuppRec_MSBL_Off(iter_K) = SuppRec_MSBL_Off(iter_K) + SuppRec;

        % RMSE
        perf_MSBL_Off = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
        RMSE_MSBL_Off(iter_K) = RMSE_MSBL_Off(iter_K) + perf_MSBL_Off;

        if PRINT == 1
            fprintf(' Offline MSBL: \n \t time = %5.2f; SuppRec = %3.2f, Ave-RMSE = %3.2f%%; Ave-Time = %4.3f \n',...
            time_elapsed,SuppRec,RMSE_MSBL_Off(iter_K)/iter*100,Time_MSBL_Off(iter_K)/iter);
        end;


%% ****************** Online Sawtooth Lag MSBL ****************** 
        for no_init = 1 : 2
            for iter_OutLen = 1 : lt_outLen
                tic;
                X = Online_MSBL(Y(:,1:SPARSE_NUM(iter_K)), A, stdnoise*eye(MES_LEN), 'delay', DELAY,...
                    'expon',0.6,'no_init',no_init-1,'out_length',OUT_LEN(iter_OutLen));
                time_elapsed = toc;
                Time_MSBL_Saw(iter_K,no_init,iter_OutLen) = Time_MSBL_Saw(iter_K,no_init,iter_OutLen ) + time_elapsed;

                % Support Recovery
                SuppRec_Saw = length(find(abs(Xgen(:,1:SPARSE_NUM(iter_K)) - X) < THRESHOLD))/(SPARSE_LEN*SPARSE_NUM(iter_K));
                SuppRec_MSBL_Saw(iter_K,no_init,iter_OutLen ) ...
                    = SuppRec_MSBL_Saw(iter_K,no_init,iter_OutLen ) + SuppRec_Saw;

                % RMSE
                perf_MSBL_Saw = (norm(Xgen(:,1:SPARSE_NUM(iter_K)) - X,'fro')/norm(Xgen(:,1:SPARSE_NUM(iter_K)),'fro'))^2;
                RMSE_MSBL_Saw(iter_K,no_init,iter_OutLen ) = RMSE_MSBL_Saw(iter_K,no_init,iter_OutLen ) + perf_MSBL_Saw;
                if PRINT == 1
                    fprintf('\n Sawtooth Lag MSBL: \n \t time = %5.2f; SuppRec = %3.2f; Ave-RMSE = %3.2f%%; Ave-Success_Rate = %4.3f%%; \n',...
                            time_elapsed,SuppRec_Saw,RMSE_MSBL_Saw(iter_K,no_init,iter_OutLen)/iter*100,...
                            SuppRec_MSBL_Saw(iter_K,no_init,iter_OutLen )/iter*100);
                end;
            end;
        end
        
        status = (iter - 1 + iter_K/lt_K)/ITER_NUM;
        waitbar(status,h,sprintf('K: %3.2f %%',status*100));
    end
    save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', ...
    'Time_MSBL_Off', 'SuppRec_MSBL_Off', 'RMSE_MSBL_Off', ...
    'Time_MSBL_Saw', 'SuppRec_MSBL_Saw', 'RMSE_MSBL_Saw');
end
    

close(h);

%% Save Results
Time_MSBL_Off    = Time_MSBL_Off/ITER_NUM;
SuppRec_MSBL_Off = SuppRec_MSBL_Off/ITER_NUM;
RMSE_MSBL_Off     = RMSE_MSBL_Off/ITER_NUM;

Time_MSBL_Saw    = Time_MSBL_Saw/ITER_NUM;
SuppRec_MSBL_Saw = SuppRec_MSBL_Saw/ITER_NUM;
RMSE_MSBL_Saw     = RMSE_MSBL_Saw/ITER_NUM;

save(OUTFILE, 'ITER_NUM', 'MES_LEN', 'SPARSE_LEN','SPARSITY', 'SNR', 'DELAY', 'SPARSE_NUM', 'OUT_LEN', ...
    'Time_MSBL_Off', 'SuppRec_MSBL_Off', 'RMSE_MSBL_Off', ...
    'Time_MSBL_Saw', 'SuppRec_MSBL_Saw', 'RMSE_MSBL_Saw');

%% Plot and save figure

% colors
DarkGreen   = [0.2       0.5000    0.2   ];
LightGreen  = [0.4660    0.6740    0.1880];
LightBlue   = [0         0.8       1     ];
SkyBlue     = [0         0.4470    0.7410];
Purple      = [0.4940    0.1840    0.5560];
DarkYellow  = [0.9290    0.6940    0.1250];
Orange      = [0.8500    0.3250    0.0980];
BrickRed    = [0.6350    0.0780    0.1840];
DarkCyan    = [0         0.6       0.6   ];
Gray        = [0.55      0.55      0.55  ];

%%%%%%% RMSE
figure;
set(0,'defaulttextinterpreter','tex')
set(0,'DefaultLineLineWidth',3)
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineMarkerSize',14)
set(0,'DefaultAxesFontWeight','bold')
set(gca,'FontSize',16)
set(get(gca,'Xlabel'),'FontSize',16)
set(get(gca,'Ylabel'),'FontSize',16)
set(get(gca,'Title'),'FontSize',16)
set(get(gca,'Xlabel'),'FontWeight','bold')
set(get(gca,'Ylabel'),'FontWeight','bold')
set(get(gca,'Title'),'FontWeight','bold')
HA = semilogy(  SPARSE_NUM, RMSE_MSBL_Saw(:,2,3),'-x' ,...
                SPARSE_NUM, RMSE_MSBL_Saw(:,2,2),'--d',...
                SPARSE_NUM, RMSE_MSBL_Saw(:,2,1),':s',...
                SPARSE_NUM, RMSE_MSBL_Saw(:,1,3),'->' ,...
                SPARSE_NUM, RMSE_MSBL_Saw(:,1,2),'--<',...
                SPARSE_NUM, RMSE_MSBL_Saw(:,1,1),':v',...
                SPARSE_NUM, RMSE_MSBL_Off,'.-'...
                );
axis square;
ylabel('RMSE');
xlabel('No. of sparse vectors K')
% h = legend( '\textbf{No Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Offline}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
set(HA(1),'color', SkyBlue);
set(HA(2),'color', Orange);         
set(HA(3),'color', DarkGreen);
set(HA(4),'color', DarkYellow);
set(HA(5),'color', Purple);
set(HA(6),'color', BrickRed);  
set(HA(7),'color', 'k'); 
% Save figures
saveas(gca,RMSE_FIG);
saveas(gca, RMSE_EPS, 'epsc');

%%%%%%% Support Recovery
figure;
set(0,'defaulttextinterpreter','tex')
HA = semilogy(  SPARSE_NUM, SuppRec_MSBL_Off,'-.',...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,1,1),':v' ,...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,1,2),'--<' ,...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,1,3),'->',...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,2,1),':s' ,...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,2,2),'--d' ,...
                SPARSE_NUM, SuppRec_MSBL_Saw(:,2,3),'-x');
axis square;
ylabel('SRR');
xlabel('No. of sparse vectors K')
% h = legend( '\textbf{Offline}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 5}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
set(HA(7),'color', SkyBlue);
set(HA(6),'color', Orange);         
set(HA(5),'color', DarkGreen);
set(HA(4),'color', DarkYellow);
set(HA(3),'color', Purple);
set(HA(2),'color', BrickRed);  
set(HA(1),'color', 'k'); 
% Save figures
saveas(gca,SUPP_FIG);
saveas(gca, SUPP_EPS, 'epsc');

%%%%%%% Run Time
figure;
set(0,'defaulttextinterpreter','tex')
HA = semilogy(  SPARSE_NUM, Time_MSBL_Off(:),'-.',...
                SPARSE_NUM, Time_MSBL_Saw(:,1,1),':v',...
                SPARSE_NUM, Time_MSBL_Saw(:,1,2),'--<',...
                SPARSE_NUM, Time_MSBL_Saw(:,1,3),'->',...
                SPARSE_NUM, Time_MSBL_Saw(:,2,1),':s' ,...
                SPARSE_NUM, Time_MSBL_Saw(:,2,2),'--d' ,...
                SPARSE_NUM, Time_MSBL_Saw(:,2,3),'-x'...
                );
set(HA(7),'color', SkyBlue);
set(HA(6),'color', Orange);         
set(HA(5),'color', DarkGreen);
set(HA(4),'color', DarkYellow);
set(HA(3),'color', Purple);
set(HA(2),'color', BrickRed);  
set(HA(1),'color', 'k');  
axis square;
ylabel('Run Time');
xlabel('No. of sparse vectors K')
% h = legend( '\textbf{Offline}',...
%             '\textbf{Init $\bar{\Delta}$= 1}',...
%             '\textbf{Init $\bar{\Delta}$= 3}',...
%             '\textbf{Init $\bar{\Delta}$= 5}',...
%             '\textbf{No Init $\bar{\Delta}$= 1}',...
%             '\textbf{No Init $\bar{\Delta}$= 3}',...
%             '\textbf{No Init $\bar{\Delta}$= 5}'...
%             );
% set(h,'Interpreter','latex', 'FontWeight','bold');
% Save figures
saveas(gca,TIME_FIG);
saveas(gca, TIME_EPS, 'epsc');
