
function [X, err] = Fixed_Online_KMSBL(Y, A, R, D, varargin)

%===============================================================================
%
%                              Online Sawtooth Lag KMSBL Algorithm
%
% ===== INPUTS =====
%   A           : m X N dictionary matrix
%
%   Y           : m X L measurement matrix
%  
%   R           : m X m noise covariance matrix
%
%   D           : N X N inter sparse vector covariance matrix
%
%===== OPTIONALS ======
%  'DELAY'       : Maximum delay between measurement and estimation
%                    [ Default value: DELAY = 0        ]
%
%  'MAX_ITERS'   : Maximum number of iterations.
%                    [ Default value: MAX_ITERS = 2000 ]
%
%  'EPSILON'     : Threshold to stop the whole algorithm. 
%                    [ Default value: EPSILON = 1e-8   ]
%
%  'PRINT'       : Display flag. If = 1: show output; If = 0: supress output
%                    [ Default value: PRINT = 0        ]
%
%  'NO_INIT'     : Proper initialization flag. If = 1: fixed initilization;
%                       If = 0: proper initilization
%                    [ Default value: NO_INIT = 0        ]
%
% ===== OUTPUTS =====
%   X          : the estimated solution matrix, or called source matrix (size: M X L)
%  err         : Error in signal covarinace
%
% *** Reference ***
% [1] Geethu Joseph, and Chandra R. Murthy, "Recursive Bayesian algorithm
% for sparse signal recovery"
%
% *** Author ***
%   Geethu Joseph, ECE Dept., IISc
%   
% *** Version ***
%   1.1 (16/02/2016)
%
  
%% Dimension of the Problem
[m1 N L1] = size(A); 
[m2 m3 L2] = size(R);
[m L] = size(Y);
[N1 N2] = size(D);

% Dimesion Check:

if m1 ~= m
    error('Dimensions of measurement vectors and matrices should match');
end;

if m2 ~= m3
    error('Covariance matrix should be square');
else 
    if m3 == 1
        R = R*eye(m);
    else 
        if m2 ~= m
            error('Dimensions of measurement vector and covariance should match');
        end;
    end;
end;

if N1 ~= N2
    error('Sparse Vector Covariance matrix should be square');
else
    if N1 == 1
        D = D*eye(N);
    else
        if N1 ~= N
        error('Dimensions of sparse vector and covariance should match');
        end;
    end;
end;

if L1 ~= 1 && L1 ~= L
      error('Dimensions of measurement vectors and matrices should match');
end;

if L2 ~= 1 && L2 ~= L
      error('Dimensions of measurement vector and covarinace should match');
end;

%% Default Control Parameters
DELAY       = 0;            % max. delay allowed
PRINT       = 0;            % don't show progress information
NO_INIT     = 0;          % fixed initilaization
UPDATE      = 1;
CONVERGE    = 0;
EXPONENT    = 1;
% Control Flags

if(mod(length(varargin),2)==1)
    error('Optional parameters should always go by pairs\n');
else
    for i=1:2:(length(varargin)-1)
        switch lower(varargin{i})
            case 'delay'   
                DELAY = varargin{i+1};
            case 'print'    
                PRINT = varargin{i+1}; 
            case 'no_init'
                NO_INIT = varargin{i+1};
            case 'update'
                UPDATE = varargin{i+1};
            case 'trueval'
                CONVERGE = 1;
                TRUE_VAL = varargin{i+1};
            case 'expon'
                EXPONENT = varargin{i+1};
            otherwise
                error(['KMSBL: Unrecognized parameter: ''' varargin{i} '''']);
        end
    end
end

if DELAY > L DELAY = L; end;
if (PRINT) fprintf('\nRunning Online KMSBL ...\n'); end

ERR_SIZE = L - DELAY;
if CONVERGE == 1 err = zeros(ERR_SIZE,1); else err = []; end

%% Initializations 

gamma       = 4*ones(N,1);                % signal covariance
mu          = zeros(N,L);

%% *** Initialization loop ***
if NO_INIT == 0
    A_required = get_matrix(A,1,DELAY+1);
    R_required = get_matrix(R,1,DELAY+1);
    [X_dummy gamma] = KMSBL(Y(:,1:DELAY+1), A_required, R_required, D, 'init', gamma);
else
    x_last  = mu(:,1);
    P_last  = diag(gamma);
    for t = 1 : DELAY + 1
        A_required = get_matrix(A,t,t);
        R_required = get_matrix(R,t,t);
        [x_last gamma_update P_last] = Interval_mean_update(Y(:,t:DELAY+1),A_required,...
                                            R_required,D,gamma,x_last,P_last,1);
        gamma = gamma + 1/t^EXPONENT*(gamma_update - gamma);
    end;
end;
if CONVERGE == 1 err(1) = norm(gamma - TRUE_VAL)^2; end
%% *** Online learning loop ***
    A_required = get_matrix(A,1,DELAY+1);
	R_required = get_matrix(R,1,DELAY+1);
    [mu_temp DUMMY P_temp] = Interval_mean_update(Y(:,1:DELAY+1),A_required,...
                                            R_required,D,gamma,mu(:,1),diag(gamma),[1 DELAY+1]);
	mu(:,1) = mu_temp(:,1);   
    for l = DELAY+2 : L
               
        gamma_old = gamma;
        mu_old = mu_temp(:,2);
        P_old = P_temp(:,:,2);
        if UPDATE == 1
            A_temp = get_matrix(A,l,l);
            A_required = A_temp*D^DELAY;
            R_required = A_temp*(eye(N)-D^(2*DELAY))*diag(gamma)*A_temp'+ get_matrix(R,l,l);
            [DUMMY gamma_update DUMMY] = Interval_mean_update(Y(:, l),...
                                                            A_required,R_required,D,gamma_old,...
                                                            mu_old,P_old,1);
            gamma = gamma_old + 1/l^EXPONENT*(gamma_update - gamma_old);
            
        end;
        l_start = l-DELAY;
        if CONVERGE == 1 err(l_start) = norm(gamma - TRUE_VAL)^2; end
        
        A_required = get_matrix(A,l_start, l);
        R_required = get_matrix(R,l_start,l);
        P_last = P_temp(:,:,1);
        [mu_temp DUMMY P_temp] = Interval_mean_update(Y(:,l_start : l),...
                                                            A_required,R_required,D,gamma,...
                                                            mu(:,l_start-1),P_last,[1 DELAY+1]);
        mu(:,l_start) = mu_temp(:,1);
        P_last = P_temp(:,:,1);
        
        %l-DELAY:l-DELAY+DELTA_BAR-1
    end
A_required = get_matrix(A,l_start+1,L);
R_required = get_matrix(R,l_start+1,L);
[mu(:,l_start+1:L) DUMMY P_temp] = Interval_mean_update(Y(:, l_start+1:L),...
                                                            A_required,R_required,D,gamma,...
                                                            mu(:,l_start),P_last,DELAY);    
X = mu; 

if (PRINT) fprintf('\nFinish running ...\n'); end

end
