
function [X, gamma] = KMSBL(Y, A, R, D, varargin)

%===============================================================================
%
%                               MSBL Algorithm
%
% ===== INPUTS =====
%   A           : m X N dictionary matrix
%
%   Y           : m X L measurement matrix
%  
%   R           : m X m noise covariance matrix
%
%   D           : N X N correlation matrix
%
%===== OPTIONALS ======
%
%  'MAX_ITERS'   : Maximum number of iterations.
%                    [ Default value: MAX_ITERS = 2000 ]
%
%  'EPSILON'     : Threshold to stop the whole algorithm. 
%                    [ Default value: EPSILON = 1e-8   ]
%
%  'PRINT'       : Display flag. If = 1: show output; If = 0: supress output
%                    [ Default value: PRINT = 0        ]
%
% ===== OUTPUTS =====
%   X          : the estimated solution matrix, or called source matrix (size: M X L)
%   gamma      : estimate of hyperparameters
%
%
% *** Reference ***
% [1] David P. Wipf, and Bhaskar D. Rao, "An empirical Bayesian strategy for solving
%     the simultaneous sparse spproximation problem, IEEE Trans. Signal
%     Processing", Vol.55, No.7, 2007.
%
% *** Author ***
%   Geethu Joseph, ECE Dept., IISc
%   
% *** Version ***
%   1.1 (12/02/2016)
%
  
%% Dimension of the Problem
[m1 N L1] = size(A); 
[m2 m3 L2] = size(R);
[m L] = size(Y);
[N1 N2] = size(D);

% Dimesion Check:

if m1 ~= m
    error('Dimensions of measurement vectors and matrices should match');
end;

if m2 ~= m3
    error('Covariance matrix should be square');
else 
    if m3 == 1
        R = R*eye(m);
    else 
        if m2 ~= m
            error('Dimensions of measurement vector and covariance should match');
        end;
    end;
end;


if N1 ~= N || N2 ~= N1
	error('Dimensions of sparse vector and correlation should match');
end;

if L1 ~= 1 && L1 ~= L
      error('Dimensions of measurement vectors and matrices should match');
end;

if L2 ~= 1 && L2 ~= L
      error('Dimensions of measurement vector and covarinace should match');
end;

%% Default Control Parameters
EPSILON     = 10^-5;        % threshold to stop
MAX_ITERS   = 2000;         % maximum iterations
PRINT       = 0;            % don't show progress information

gamma       = 4*ones(N,1);    % signal covariance 
% Control Flags

if(mod(length(varargin),2)==1)
    error('Optional parameters should always go by pairs\n');
else
    for i=1:2:(length(varargin)-1)
        switch lower(varargin{i})
            case 'epsilon'   
                EPSILON = varargin{i+1}; 
            case 'print'    
                PRINT = varargin{i+1}; 
            case 'max_iters'
                MAX_ITERS = varargin{i+1};
            case 'init'
                gamma = varargin{i+1};
            otherwise
                error(['MSBL: Unrecognized parameter: ''' varargin{i} '''']);
        end
    end
end



if (PRINT) fprintf('\nRunning MSBL ...\n'); end


%% Initializations 

mu         = zeros(N,L);
P          = 4*eye(N);
count      = 0;                        % iteration count
C          = zeros(N,L);
C1         = zeros(N,1);

%% *** Learning loop ***
while (1)

    mu_old  = mu;
    P_old   = P;
    Gamma   = diag(gamma);
    G       = diag(sqrt(gamma));
    
    % Compute state statistics
    
    [mu P P1] = Fixed_Interval(Y, A, R, D, gamma, mu_old(:,1), P_old(:,:,1));  
                
    gamma_old = gamma;
    
    C = mu.^2;
    C(:,1) = C(:,1) +  diag(P(:,:,1));
    gamma = zeros(N,1);
    for t = 2 : L
        C(:,t) = C(:,t) + diag(P(:,:,t));
        C1 = diag(P1(:,:,t-1)) + mu(:,t-1).*mu(:,t);
        gamma  = gamma + C(:,t) + D^2*C(:,t-1) - 2*D*C1;
    end
    gamma = 1/L*( (eye(N) - D^2)^-1 *gamma + C(:,1));
  	
	% *** Check stopping conditions, etc. ***
	count = count + 1;

    if (PRINT) 
    	disp(['iters: ',num2str(count),'   num coeffs: ',num2str(m), ...
                '   gamma change: ',num2str(max(abs(gamma - gamma_old)))]); 
	end;

	if (count >= MAX_ITERS)	break;  end;

    change = norm(gamma - gamma_old);
	if (change < EPSILON)   break;  end;
end;


% Expand hyperparameters 
X = mu; 

if (PRINT) fprintf('\nFinish running ...\n'); end
return;
