
function [X, gamma] = MSBL(Y, A, R, varargin)

%===============================================================================
%
%                               MSBL Algorithm
%
% ===== INPUTS =====
%   A           : m X N dictionary matrix
%
%   Y           : m X L measurement matrix
%  
%   R           : m X m noise covariance matrix
%
%===== OPTIONALS ======
%
%  'MAX_ITERS'   : Maximum number of iterations.
%                    [ Default value: MAX_ITERS = 2000 ]
%
%  'EPSILON'     : Threshold to stop the whole algorithm. 
%                    [ Default value: EPSILON = 1e-8   ]
%
%  'PRINT'       : Display flag. If = 1: show output; If = 0: supress output
%                    [ Default value: PRINT = 0        ]
%  'INIT'         : Initilization for the signal covariance
%                    [ Default value: 4I               ]
%
% ===== OUTPUTS =====
%   X          : the estimated solution matrix, or called source matrix (size: M X L)
%   gamma      : estimate of hyperparameters
%
%
% *** Reference ***
% [1] David P. Wipf, and Bhaskar D. Rao, "An empirical Bayesian strategy for solving
%     the simultaneous sparse spproximation problem, IEEE Trans. Signal
%     Processing", Vol.55, No.7, 2007.
%
% *** Author ***
%   Geethu Joseph, ECE Dept., IISc
%   
% *** Version ***
%   1.1 (12/02/2016)
%
  
%% Dimension of the Problem
[m1 N L1] = size(A); 
[m2 m3 L2] = size(R);
[m L] = size(Y);

% Dimesion Check:

if m1 ~= m
    error('Dimensions of measurement vectors and matrices should match');
end;

if m2 ~= m3
    error('Covariance matrix should be square');
else 
    if m3 == 1  R = R*eye(m,m);
    else if m2 ~= m
            error('Dimensions of measurement vector and covariance should match');
        end;
    end;
end;


if L1 == 1  COMMON_A = 1;
else    
    if L == L1  COMMON_A = 0;
    else
      error('Dimensions of measurement vectors and matrices should match');
    end;
end;

if L2 == 1  COMMON_R = 1;
else
    if L == L2  COMMON_R = 0;
    else
      error('Dimensions of measurement vector and covarinace should match');
    end;
end;

%% Default Control Parameters
EPSILON     = 10^-8;        % threshold to stop
MAX_ITERS   = 2000;         % maximum iterations
PRINT       = 0;            % don't show progress information

gamma       = ones(N,1);    % signal covariance 
% Control Flags

if(mod(length(varargin),2)==1)
    error('Optional parameters should always go by pairs\n');
else
    for i=1:2:(length(varargin)-1)
        switch lower(varargin{i})
            case 'epsilon'   
                EPSILON = varargin{i+1}; 
            case 'print'    
                PRINT = varargin{i+1}; 
            case 'max_iters'
                MAX_ITERS = varargin{i+1};
            case 'init'
                gamma = varargin{i+1};
            otherwise
                error(['MSBL: Unrecognized parameter: ''' varargin{i} '''']);
        end
    end
end



if (PRINT) fprintf('\nRunning MSBL ...\n'); end


%% Initializations 

mu          = zeros(N,L);
P          = zeros(N,L);
count       = 0;                        % iteration count


%% *** Learning loop ***
while (1)

    mu_old  = mu;
    Gamma   = diag(gamma);
    G       = diag(sqrt(gamma));
    
    % Compute state statistics
    
    for l = 1 : L
                
        if COMMON_A == 1    A_l = A;
        else    A_l = A(:,:,l);     end;
        
        
        if COMMON_R == 1    R_l = R;
        else    R_l = R(:,:,l);     end;
        
        
        if norm(R_l) ~= 0
            J = Gamma*A_l'*(A_l*Gamma*A'+R)^-1;
        else
            J = G*pinv(A_l*G);
        end;
        
        mu(:,l) = J* Y(:,l);
        P(:,l)  = diag( (eye(N,N) - J*A_l)*Gamma);
    end;
   

	% *** Update hyperparameters, i.e. Eq(18) in the reference ***
  	gamma_old = gamma;
	mu2_bar = sum(abs(mu).^2,2);
    P_bar = sum(abs(P),2);
    
	gamma = 1/L*(mu2_bar + P_bar);

	% *** Check stopping conditions, etc. ***
	count = count + 1;

    if (PRINT) 
    	disp(['iters: ',num2str(count),'   num coeffs: ',num2str(m), ...
                '   gamma change: ',num2str(max(abs(gamma - gamma_old)))]); 
	end;

	if (count >= MAX_ITERS)	break;  end;

    change = norm(gamma - gamma_old);
	if (change < EPSILON)   break;  end;
end;


% Expand hyperparameters 
X = mu; 

if (PRINT) fprintf('\nFinish running ...\n'); end
return;
