
function [X, err] = Online_MSBL(Y, A, R, varargin)

%===============================================================================
%
%                              Online KMSBL Algorithm
%
% ===== INPUTS =====
%   A           : m X N dictionary matrix
%
%   Y           : m X L measurement matrix
%  
%   R           : m X m noise covariance matrix
%
%===== OPTIONALS ======
%  'DELAY'       : Maximum delay between measurement and estimation
%                    [ Default value: DELAY = 0        ]
%
%  'MAX_ITERS'   : Maximum number of iterations.
%                    [ Default value: MAX_ITERS = 2000 ]
%
%  'EPSILON'     : Threshold to stop the whole algorithm. 
%                    [ Default value: EPSILON = 1e-8   ]
%
%  'PRINT'       : Display flag. If = 1: show output; If = 0: supress output
%                    [ Default value: PRINT = 0        ]
%  'NO INIT'     : Initialzation flag. If = 1: fixed intilization of 4I;
%                                       If = 0: initilization using Offline algorithm
%
%  'OUT_LENGTH'  : Length of output block  
%                   [ Default value: DELTA_BAR = 1      ]
%  'EXPON'       :  Exponent of learning rate
%                   [ Default value: EXPONENT = 1      ]
%  ''
% ===== OUTPUTS =====
%   X          : the estimated solution matrix, or called source matrix (size: M X L)
%   gamma_ind  : indexes of nonzero gamma_i
%   count      : no. of EM iterations
%
%
% *** Reference ***
% [1] Geethu Joseph, and Chandra R. Murthy, "Recursive Bayesian algorithm
% for sparse signal recovery"
%
% *** Author ***
%   Geethu Joseph, ECE Dept., IISc
%   
% *** Version ***
%   1.1 (16/02/2016)
%
  
%% Dimension of the Problem
[m1 N L1] = size(A); 
[m2 m3 L2] = size(R);
[m L] = size(Y);

% Dimesion Check:

if m1 ~= m
    error('Dimensions of measurement vectors and matrices should match');
end;

if m2 ~= m3
    error('Covariance matrix should be square');
else 
    if m3 == 1
        R = R*eye(m);
    else 
        if m2 ~= m
            error('Dimensions of measurement vector and covariance should match');
        end;
    end;
end;


if L1 ~= 1 && L1 ~= L
      error('Dimensions of measurement vectors and matrices should match');
end;

if L2 ~= 1 && L2 ~= L
      error('Dimensions of measurement vector and covarinace should match');
end;

%% Default Control Parameters
DELAY       = 0;            % max. delay allowed
PRINT       = 0;            % don't show progress information
NO_INIT     = 0;          % fixed initilaization
UPDATE      = 1;
CONVERGE    = 0;
EXPONENT    = 1;
AVERAGE     = 0;
FIXED_SAW   = 1;
DELTA_BAR   = 1;
% Control Flags

if(mod(length(varargin),2)==1)
    error('Optional parameters should always go by pairs\n');
else
    for i=1:2:(length(varargin)-1)
        switch lower(varargin{i})
            case 'delay'   
                DELAY = varargin{i+1};
            case 'print'    
                PRINT = varargin{i+1}; 
            case 'no_init'
                NO_INIT = varargin{i+1};
            case 'update'
                UPDATE = varargin{i+1};
            case 'trueval'
                CONVERGE = 1;
                TRUE_VAL = varargin{i+1};
            case 'expon'
                EXPONENT = varargin{i+1};
            case 'out_length'
                DELTA_BAR = varargin{i+1};
            otherwise
                error(['Online MSBL: Unrecognized parameter: ''' varargin{i} '''']);
        end
    end
end

if FIXED_SAW == 0 && DELTA_BAR ~= 1
    error('Output length should be 1 for fixed lag scheme');
end;

   
if DELAY > L DELAY = L; end;
if (PRINT) fprintf('\nRunning Online KMSBL ...\n'); end

ERR_SIZE = floor((L-DELAY)/DELTA_BAR);
if CONVERGE == 1 err = zeros(ERR_SIZE,1); else err = []; end

%% Initializations 

gamma       = ones(N,1);                % signal covariance
mu          = zeros(N,L);

%% *** Initialization loop ***
DELAY_temp = min (DELAY+1,L);
if NO_INIT == 0
    A_required = get_matrix(A,1,DELAY_temp);
    R_required = get_matrix(R,1,DELAY_temp);
    [X_dummy gamma] = MSBL( Y(:,1:DELAY_temp), A_required, R_required, 'init',gamma);
else
    for t = 1 : DELAY_temp
        A_required = get_matrix(A,t,t);
        R_required = get_matrix(R,t,t);
        [DUMMY gamma_update] = get_mean_update(Y(:,t),A_required,R_required,gamma);
        gamma =gamma + 1/t^EXPONENT*(gamma_update - gamma);
    end;
end;
A_required = get_matrix(A,1,DELTA_BAR);
R_required = get_matrix(R,1,DELTA_BAR);
mu(:,1:DELTA_BAR) = get_mean_update(Y(:,1 : DELTA_BAR),...
                                        A_required,R_required,gamma);
lt_end = DELTA_BAR;
%% *** Online learning loop ***
l_t = 0;
    for l = DELAY + DELTA_BAR + 1  : DELTA_BAR : ceil((L -DELAY-1)/DELTA_BAR)*DELTA_BAR + DELAY + 1
                  
        if CONVERGE == 1
            l_t = l_t + 1;
            err(l_t) = norm(gamma - TRUE_VAL)^2;
        end
        
        gamma_old = gamma;
        l_len = min(l,L);
        if UPDATE == 1
            A_required = get_matrix(A,l-DELTA_BAR+1,l_len);
            R_required = get_matrix(R,l-DELTA_BAR+1,l_len);
            [DUMMY gamma_update] = get_mean_update(Y(:, l-DELTA_BAR+1 : l_len),...
                                                            A_required,R_required,gamma_old);
            gamma = gamma_old + 1/l_len^EXPONENT*(gamma_update - DELTA_BAR*gamma_old);
        end;
        l_end = min(l-DELAY+DELTA_BAR-1,L);
        A_required = get_matrix(A,l-DELAY,l_end);
        R_required = get_matrix(R,l-DELAY,l_end);
        mu(:,l-DELAY:l_end) = get_mean_update(Y(:,l-DELAY:l_end),...
                                        A_required,R_required,gamma_old);
	end
    
if CONVERGE == 1
	l_t = l_t + 1;
	err(l_t) = norm(gamma - TRUE_VAL)^2;
end

A_required = get_matrix(A,lt_end+1,L);
R_required = get_matrix(R,lt_end+1,L);
mu(:,lt_end+1:L) = get_mean_update(Y(:,lt_end+1 :L),...
                                        A_required,R_required,gamma);
X = mu; 

if (PRINT) fprintf('\nFinish running ...\n'); end

end

function [mu update] = get_mean_update(Y,A,R,gamma)
    
    L = size(Y,2);
    N = size(A,2);
    mu = zeros(N,L);
    update = zeros(N,1);
    
    Gamma = diag(gamma);
    for t = 1 : L
        A_t = get_matrix(A,t,t);
        R_t = get_matrix(R,t,t);

        if norm(R_t) ~= 0
            J = Gamma*A_t'*(A_t*Gamma*A_t'+R_t)^-1;
        else
            J = Gamma^0.5*pinv(A_t*Gamma^0.5);
        end;
        P = diag((eye(N) - J*A_t)).*gamma;
        mu(:,t) = J* Y(:,t);
        update = update + P +  mu(:,t).* mu(:,t);
    end;
end