
function [X, err] = Sawtooth_Online_KMSBL(Y, A, R, D, varargin)

%===============================================================================
%
%                              Online Sawtooth Lag KMSBL Algorithm
%
% ===== INPUTS =====
%   A           : m X N dictionary matrix
%
%   Y           : m X L measurement matrix
%  
%   R           : m X m noise covariance matrix
%
%   D           : N X N inter sparse vector covariance matrix
%
%===== OPTIONALS ======
%  'DELAY'       : Maximum delay between measurement and estimation
%                    [ Default value: DELAY = 0        ]
%
%  'MAX_ITERS'   : Maximum number of iterations.
%                    [ Default value: MAX_ITERS = 2000 ]
%
%  'EPSILON'     : Threshold to stop the whole algorithm. 
%                    [ Default value: EPSILON = 1e-8   ]
%
%  'PRINT'       : Display flag. If = 1: show output; If = 0: supress output
%                    [ Default value: PRINT = 0        ]
%
%  'NO_INIT'     : Proper initialization flag. If = 1: fixed initilization;
%                       If = 0: proper initilization
%                    [ Default value: NO_INIT = 0        ]
%
% ===== OUTPUTS =====
%   X          : the estimated solution matrix, or called source matrix (size: M X L)
%  err         : Error in signal covarinace
%
% *** Reference ***
% [1] Geethu Joseph, and Chandra R. Murthy, "Recursive Bayesian algorithm
% for sparse signal recovery"
%
% *** Author ***
%   Geethu Joseph, ECE Dept., IISc
%   
% *** Version ***
%   1.1 (16/02/2016)
%
  
%% Dimension of the Problem
[m1 N L1] = size(A); 
[m2 m3 L2] = size(R);
[m L] = size(Y);
[N1 N2] = size(D);

% Dimesion Check:

if m1 ~= m
    error('Dimensions of measurement vectors and matrices should match');
end;

if m2 ~= m3
    error('Covariance matrix should be square');
else 
    if m3 == 1
        R = R*eye(m);
    else 
        if m2 ~= m
            error('Dimensions of measurement vector and covariance should match');
        end;
    end;
end;

if N1 ~= N2
    error('Sparse Vector Covariance matrix should be square');
else
    if N1 == 1
        D = D*eye(N);
    else
        if N1 ~= N
        error('Dimensions of sparse vector and covariance should match');
        end;
    end;
end;

if L1 ~= 1 && L1 ~= L
      error('Dimensions of measurement vectors and matrices should match');
end;

if L2 ~= 1 && L2 ~= L
      error('Dimensions of measurement vector and covarinace should match');
end;

%% Default Control Parameters
DELAY       = 0;            % max. delay allowed
PRINT       = 0;            % don't show progress information
NO_INIT     = 0;          % fixed initilaization
UPDATE      = 1;
CONVERGE    = 0;
EXPONENT    = 1;
DELTA_BAR   = 1;
% Control Flags

if(mod(length(varargin),2)==1)
    error('Optional parameters should always go by pairs\n');
else
    for i=1:2:(length(varargin)-1)
        switch lower(varargin{i})
            case 'delay'   
                DELAY = varargin{i+1};
            case 'print'    
                PRINT = varargin{i+1}; 
            case 'no_init'
                NO_INIT = varargin{i+1};
            case 'update'
                UPDATE = varargin{i+1};
            case 'trueval'
                CONVERGE = 1;
                TRUE_VAL = varargin{i+1};
            case 'expon'
                EXPONENT = varargin{i+1};
            case 'out_length'
                DELTA_BAR = varargin{i+1};
            otherwise
                error(['KMSBL: Unrecognized parameter: ''' varargin{i} '''']);
        end
    end
end

if DELTA_BAR > DELAY+1
     error('Output length can not exceed delay');
end;
    
if DELAY > L DELAY = L; end;
if (PRINT) fprintf('\nRunning Online KMSBL ...\n'); end

ERR_SIZE = ceil(L/DELTA_BAR);
if CONVERGE == 1 err = zeros(ERR_SIZE,1); else err = []; end

%% Initializations 

X = zeros(N,L);
gamma       = 4*ones(N,1);                % signal covariance


%% *** Initialization loop ***
A_required = get_matrix(A,1,DELAY+1);
R_required = get_matrix(R,1,DELAY+1);
[X_dummy gamma] = KMSBL(Y(:,1:DELAY+1), A_required, R_required, D, 'init', gamma);

Supp = find(gamma > 0.0005);

mu          = zeros(length(Supp),L);
%% *** Online learning loop ***
   
    
    for l = 1 : L
        
        A_temp = get_matrix(A,1,l);
        A_required = A_temp(:,Supp);
        R_required = get_matrix(R,1,l);
        
        mu(:,l) = pinv(A_required)*Y(:,l);
    end
    
X(Supp,:) = mu; 

if (PRINT) fprintf('\nFinish running ...\n'); end

end
